#define DK_MAGIC        0xf1e2d3c4      /* magic number for all structures */
#define DK_XMAGIC       0xc4d3e2f1      /* magic number for index structure */
#define DK_NINDEX       4               /* # indexes per disk */
#define DK_NCOPIES      8               /* max # data structure copies */
#define DK_STRUCTSIZE   2048            /* min size of the on disk structures */
#define DK_MAXSECTOR    4096            /* max supported sector size *
                                         * smaller sectors are okay  */
#define DK_IDLEN        64              /* max length of the id string */

struct dkaddr {
        uint32_t cylinder;
        uint32_t head;
        uint32_t sector;
};


struct dkindex_ondisk {
        uint32_t dk_version;
        uint32_t dk_magic;
        union {
                struct dkindex_v1 {
                        char dkx_id[DK_IDLEN];/* unique, per device id string */
                        struct dkaddr dkx_asm[DK_NCOPIES];  /* alternate sector map */
			struct dkaddr dkx_type[DK_NCOPIES]; /* struct dktype */
			struct dkaddr dkx_part[DK_NCOPIES]; /* struct dkpart */
			struct dkaddr dkx_tune[DK_NCOPIES]; /* struct dktune */
			struct dkaddr dkx_misc[DK_NCOPIES]; /* reserved */
                } dkx_v1;

                char fill[DK_STRUCTSIZE - 3*sizeof(uint32_t)];
        };
        uint32_t dk_level;    /* level which can change within a version
			         without causing the read_index to fail */
};

#define DKNAMESIZE      28
#define NODENAMESZ      24

struct  dktype {
        uint32_t dkt_cylinders;      /* number of cylinders (total) */
        uint32_t dkt_reserved;       /* number of reserved cylinders */
        uint32_t dkt_usercyl;        /* cylinders available for user data */
        uint32_t dkt_heads;          /* number of heads per cylinder */
        uint32_t dkt_sectors;        /* number of sectors per track */
        uint16_t dkt_bsize;             /* sector size (bytes) */
        uint8_t  dkt_gap1;               /* number of bytes in gap1 */
        uint8_t  dkt_gap2;               /* number of bytes in gap2 */
        uint32_t dkt_rpm;            /* spin rate (revolutions/minute) */
        uint32_t dkt_format;         /* disk header/map format and flags */
        char     dkt_name[DKNAMESIZE];   /* name of this drive type */
        /* these are returned by ioctl for the currently opened disk */
        uint64_t dkt_size;               /* size of partition */
        char     dkt_dkname[NODENAMESZ]; /* from dk_name "CCCCN/XdiskNNNN\0" */
        uint16_t dkt_type;              /* IOCDISK or IOPDISK (elog.h) */
        uint16_t dkt_cont;              /* controller number */
        uint16_t dkt_unit;              /* driver unit number */
        uint16_t dkt_dk;                /* system dk number */
        uint32_t dkt_iodev;          /* 3 byte hardware address */
        uint32_t dkt_partno;         /* partition number */
        uint32_t dkt_offset;             /* block offset of partition */
        uint32_t dkt_maxbsize;       /* maximum single io size */
        uint32_t dkt_prefbsize;      /* prefered single io size */
        int      dkt_nreads;             /* number of read requests */
        int      dkt_nwrites;            /* number of write requests */
        int      dkt_iosize;             /* application I/O size */
        int      dkt_magic2;             /* SNI: additional magic number
                                           for sdi sdi_newconfig        */
};

struct dktype_ondisk {
        uint32_t      dk_version;
        uint32_t      dk_magic;
        union {
                struct dktype dkt_v1;
                char fill[DK_STRUCTSIZE - 3*sizeof(uint32_t)];
        } u;
        uint32_t dk_level;    /* level which can change within a version
                                 * without causing the read_type to fail */
};

#define DKPARTNAME      28              /* size for dkp_name[] */
#define DKMAXPARTS      16              /* number of partitions */

struct dkpart {
        uint32_t dkp_flags;          /* misc flags (TBD) */
        /* General information (not mandatory) */
        uint32_t dkp_pcylinders;     /* number of physical cylinders */
        uint32_t dkp_ucylinders;     /* number of user cylinders */
        uint32_t dkp_heads;          /* number of heads per cylinder */
        uint32_t dkp_sectors;        /* number of sectors per track */
        char     dkp_name[DKPARTNAME];   /* name of this partition table */
        /* Partition definition (mandatory) */
        uint32_t dkp_cylsize;        /* cylinder size in 512-byte units */
        struct {
                uint32_t dkp_offset; /* starting cylinder of partition */
                uint32_t dkp_length; /* number of cylinders in partition */
        } dkp_partition[DKMAXPARTS];
};

struct dkpart_ondisk {
        uint32_t dk_version;
        uint32_t dk_magic;
        union {
                struct dkpart dkp_v1;
                char fill[DK_STRUCTSIZE - 3*sizeof(uint32_t)];
        };
        uint32_t dk_level;    /* level which can change within a version
                                 * without causing the read_part to fail */
};

/*
 * Disk tuning structure - these are stored on the disk itself.
 * All skew and gap parameters are in units of 512 bytes.
 * Timeout parameters are in units of usec.
 */
struct dktune {
        uint32_t dku_flags;
        uint16_t dku_trkskew;   /* physical head skew */
        uint16_t dku_cylskew;   /* physical cylinder skew */
        uint16_t dku_rotgap;    /* rotational gap between all requests */
        uint16_t dku_trkgap;    /* rotational gap on track switches */
        uint16_t dku_cylgap;    /* rotational gap on cylinder switches */
        uint16_t dku_rotdelay;  /* # milliseconds of filesystem delay */
        uint32_t dku_timeout[4];/* queue timeouts */
        uint16_t dku_velocity;  /* max head velocity when seeking */
};

struct dktune_ondisk {
        uint32_t dk_version;
        uint32_t dk_magic;
        union {
                struct dktune dku_v1;
                /* leave large so it can grow easily */
                char fill[DK_MAXSECTOR - 3*sizeof(uint32_t)];
        } u;
        uint32_t dk_level;    /* level which can change within a version
                                 * without causing the read_tune to fail */
};

